// ------------------------------------------------------------------------------
//
// JavaFish.java                                          Ulrich Mller 05.06.02
//                                                        PCS1700     JBuilder 5
// Zugriffsfunktionen auf die fischertechnik Interfaces
// bentigt javaFish.DLL und umFish20.DLL mit WinRT.SYS
// Erweitert die Version vom 28.10.01, das Exception-Handling fehlt aber
//           immer noch
//
// JavaFish(ftInstanz) : Konstruktor, Angabe der (internen)Interface (0-3)
//                       optional, nur erforderlichen bei mehreren Instanzen
// setAnalogScan       : true, die Analogeingnge EX (0) und EY (1) werden
//                       gescannt (default = false).
// setSlave            : true, am Intelligenten Interface ist ein
//                       Extensionmodul angeschlossen (default = false).
// setPollInterval     : Zeit (msec) in der das Interface abgefragt wird
// setLPTAnalog        : paralleles Interface, Faktor fr die A-Eingnge
// setLPTDelay         : paralleses Interface, Faktor fr die E-Eingnge
// ---                 : OpenInterface setzt default-Werte, die Set meit
//                       erbrigen.
// openInterface       : Herstellen der Verbindung zum Interface
//                       LPT, LPT1 - 3, COM1 - 8
// closeInterface      : Schlieen der Verbindung zum Interface
// getVersion          : Version der eingesetzten javaFish.DLL
// showStatus          : KonsolAusgabe der akt. Daten
//
// setMotor            : Schalten der M-Ausgnge M1-M4(M8), (1-4(8))
//                       Linkslauf = 1, Rechtlauf = 2, Ein = 1, Aus = 0
// clearMotors         : Ausschalten aller M-Ausgnge
// setLamp             : Schalten der M-Ausgnge in "Lampenschaltung"
//                       (ein Bein an Erde) 1-8(16), true/false
// getAnalog           : Auslesen eines A-Einganges EX(0) /EY(1)
// getInput            : Auslesen eines E-Einganges E1-E8(E16) 1-8(16), true/false
// getOutputs          : Auslesen aller M-Ausgnge M1-M4(M8), M1 = niedrigstes bit
// setOutputs          : Setzen aller M-Ausgnge M1-M4(M8), M1 = niedrigstes bit
// getInputs           : Auslesen aller E-Eingnge E1-E8(E16), E1 = niedrigstes bit
// clearCounters       : Lschen aller Counter ( = 0)
// setCounter          : Setzen eines Counters auf einen Anfangswert
// getCounter          : Auslesen eines aktuellen Counterwertes
// pause               : Anhalten des Programmablaufs um n mSekunden.
// finish              : Feststellen Ende-Wunsch (ESC-Taste oder E-Eingang)
// emergencyHalt       : Anmelden eines Abbruchwunsches
// waitForInput        : Warten auf InputNr = OnOff
// waitForHigh         : Warten auf einen false/true-Durchgang von InputNr
// waitForLow          : Warten auf einen true/false-Durchgang von InputNr
//
// ------------------------------------------------------------------------------
package ftcomputing;
public class JavaFish
{
  private native int jcOpenInterface(int iNr, String PortName);
  private native int jcCloseInterface(int iNr);
  private native int jcGetInput(int iNr, int InputNr);
  private native int jcSetMotor(int iNr, int MotorNr, int Direction);
  private native int jcSetLamp(int iNr, int LampNr, int OnOff);
  private native int jcGetDCB(int iNr, int Index);
  private native void jcSetDCB(int iNr, int Index, int Wert);
  private native void jcSleep(int MilliSec);
  private native boolean jcEscape();
  private int iNr;

  static{
    try {
      System.loadLibrary("javaFish");
    }
    catch (UnsatisfiedLinkError ex){
      System.out.println("javaFish.DLL konnte nicht geladen werden");
    }
  }
  public JavaFish(){
    iNr = 0;
  }
  public JavaFish(int ftInstanz){
    iNr = (ftInstanz < 0 || ftInstanz > 3) ? 1 : ftInstanz;
  }
  public static final int OFF = 0;
  public static final int ON = 1;
  public static final int LEFT = 1;
  public static final int RIGHT = 2;
  public static final int FT_ERROR = 131071;

  public boolean emergencyHalt = false;

  public void setAnalogScan(boolean OnOff){
    if(OnOff) jcSetDCB(iNr,7,1); else jcSetDCB(iNr,7,0);
  }
  public void setSlave(boolean OnOff){
    if(OnOff) jcSetDCB(iNr,4,16); else jcSetDCB(iNr,4,8);
  }
  public void setPollInterval(int PollInterval){
    jcSetDCB(iNr, 6, PollInterval);
  }
  public void setLPTAnalog(int LPTAnalog){
    jcSetDCB(iNr, 3, LPTAnalog);
  }
  public void setLPTDelay(int LPTDelay){
    jcSetDCB(iNr, 2, LPTDelay);
  }
  public int openInterface(String PortName){
    emergencyHalt = false;
    return(jcOpenInterface(iNr, PortName));
  }
  public int closeInterface(){
    return(jcCloseInterface(iNr));
  }
  public native int getVersion();
  public void showStatus(){
    boolean res;
    System.out.println("Version      : " + getVersion());
    System.out.println("LPTAnalog    : " + jcGetDCB(iNr,3));
    System.out.println("LPTDelay     : " + jcGetDCB(iNr,2));
    if (jcGetDCB(iNr,4) == 8) res = false; else res = true;
    System.out.println("Slave        : " + res);
    System.out.println("PollInterval : " + jcGetDCB(iNr,6));
    if (jcGetDCB(iNr,7) == 0) res = false; else res = true;
    System.out.println("AnalogScan   : " + res);
    System.out.println("OutputStatus : " + jcGetDCB(iNr,8));
    System.out.println("InputStatus  : " + jcGetDCB(iNr,9));
    System.out.println("EX           : " + jcGetDCB(iNr,10));
    System.out.println("EY           : " + jcGetDCB(iNr,11));
  }
  public int setMotor(int MotorNr, int Direction){
    return(jcSetMotor(iNr, MotorNr, Direction));
  }
  public void clearMotors(){
    jcSetDCB(iNr,8,0);
  }
  public int setLamp(int LampNr, int OnOff){
    return(jcSetLamp(iNr, LampNr, OnOff));
  }
  public int getAnalog(int AnalogNr){
    return(jcGetDCB(iNr, AnalogNr+10));
  }
  public boolean getInput(int InputNr){
    return (jcGetInput(iNr, InputNr) != 0) ? true : false;
  }
  public int getOutputs(){
    return(jcGetDCB(iNr,8));
  }
  public void setOutputs(int M){
    jcSetDCB(iNr, 8, M);
  }
  public int getInputs(){
    return(jcGetDCB(iNr,9));
  }
  public void clearCounters(){
    for(int i = 12; i > 28; i++) jcSetDCB(iNr, i, 0);
  }
  public void setCounter(int CounterNr, int CounterValue){
    jcSetDCB(iNr, CounterNr+12, CounterValue);
  }
  public int getCounter(int CounterNr){
    return(jcGetDCB(iNr, CounterNr+12));
  }
  public void pause(int MilliSec){
    int dZeit = MilliSec;
    while(dZeit>100) {
      if(emergencyHalt | jcEscape()) return;
      jcSleep(100);
      dZeit -= 100;
    }
    jcSleep(dZeit);
  }
  public boolean finish(int InputNr){
    return ( jcEscape() | emergencyHalt | getInput(InputNr)) ? true : false;
  }
  public boolean finish(){
    return (jcEscape() | emergencyHalt) ? true : false;
  }

  public void waitForInput(int InputNr){
    waitForInput(InputNr, true);
  }
  public void waitForInput(int InputNr, boolean OnOff){
    do {
      if(jcEscape() | emergencyHalt) return;
    } while((jcGetInput(iNr, InputNr) == 0) == OnOff);
  }

  public void waitForHigh(int InputNr){
    waitForInput(InputNr, false);
    waitForInput(InputNr, true);
  }
  public void waitForLow(int InputNr){
    waitForInput(InputNr, true);
    waitForInput(InputNr, false);
  }
}
